/*
 * Copyright 2010 Google Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */
/*
 * This file was generated.
 *  with google-apis-code-generator 1.4.1 (build: 2013-08-14 15:32:06 UTC)
 *  on 2013-08-19 at 21:36:30 UTC 
 */

package com.google.api.services.civicinfo;

import com.google.api.client.googleapis.GoogleUtils;
import com.google.api.client.googleapis.services.AbstractGoogleClientRequest;
import com.google.api.client.googleapis.services.GoogleClientRequestInitializer;
import com.google.api.client.googleapis.services.json.AbstractGoogleJsonClient;
import com.google.api.client.http.HttpRequestInitializer;
import com.google.api.client.http.HttpTransport;
import com.google.api.client.json.JsonFactory;
import com.google.api.client.json.JsonObjectParser;
import com.google.api.client.json.JsonString;
import com.google.common.base.Preconditions;

/**
 * Service definition for CivicInfo (us_v1).
 *
 * <p>
 * An API for accessing civic information.
 * </p>
 *
 * <p>
 * For more information about this service, see the
 * <a href="https://developers.google.com/civic-information" target="_blank">API Documentation</a>
 * </p>
 *
 * <p>
 * This service uses {@link CivicInfoRequestInitializer} to initialize global parameters via its
 * {@link Builder}.
 * </p>
 *
 * <p>
 * Upgrade warning: this class now extends {@link AbstractGoogleJsonClient}, whereas in prior
 * version 1.8 it extended {@link com.google.api.client.googleapis.services.GoogleClient}.
 * </p>
 *
 * @since 1.3
 * @author Google, Inc.
 */
@SuppressWarnings("javadoc")
public class CivicInfo extends AbstractGoogleJsonClient {

  // Note: Leave this static initializer at the top of the file.
  static {
    Preconditions.checkState(GoogleUtils.VERSION.equals("1.13.2-beta"),
        "You are currently running with version %s of google-api-client. " +
        "You need version 1.13.2-beta of google-api-client to run version " +
        "1.13.2-beta of the Google Civic Information API library.", GoogleUtils.VERSION);
  }

  /**
   * The default encoded root URL of the service. This is determined when the library is generated
   * and normally should not be changed.
   *
   * @since 1.7
   */
  public static final String DEFAULT_ROOT_URL = "https://www.googleapis.com/";

  /**
   * The default encoded service path of the service. This is determined when the library is
   * generated and normally should not be changed.
   *
   * @since 1.7
   */
  public static final String DEFAULT_SERVICE_PATH = "civicinfo/us_v1/";

  /**
   * The default encoded base URL of the service. This is determined when the library is generated
   * and normally should not be changed.
   * @deprecated (scheduled to be removed in 1.13)
   */
  @Deprecated
  public static final String DEFAULT_BASE_URL = DEFAULT_ROOT_URL + DEFAULT_SERVICE_PATH;

  /**
   * Constructor.
   *
   * <p>
   * Use {@link Builder} if you need to specify any of the optional parameters.
   * </p>
   *
   * @param transport HTTP transport
   * @param jsonFactory JSON factory
   * @param httpRequestInitializer HTTP request initializer or {@code null} for none
   * @since 1.7
   */
  public CivicInfo(HttpTransport transport, JsonFactory jsonFactory,
      HttpRequestInitializer httpRequestInitializer) {
    super(transport,
        jsonFactory,
        DEFAULT_ROOT_URL,
        DEFAULT_SERVICE_PATH,
        httpRequestInitializer,
        false);
  }

  /**
   * @param transport HTTP transport
   * @param httpRequestInitializer HTTP request initializer or {@code null} for none
   * @param rootUrl root URL of the service
   * @param servicePath service path
   * @param jsonObjectParser JSON object parser
   * @param googleClientRequestInitializer Google request initializer or {@code null} for none
   * @param applicationName application name to be sent in the User-Agent header of requests or
   *        {@code null} for none
   * @param suppressPatternChecks whether discovery pattern checks should be suppressed on required
   *        parameters
   */
  CivicInfo(HttpTransport transport,
      HttpRequestInitializer httpRequestInitializer,
      String rootUrl,
      String servicePath,
      JsonObjectParser jsonObjectParser,
      GoogleClientRequestInitializer googleClientRequestInitializer,
      String applicationName,
      boolean suppressPatternChecks) {
    super(transport,
        httpRequestInitializer,
        rootUrl,
        servicePath,
        jsonObjectParser,
        googleClientRequestInitializer,
        applicationName,
        suppressPatternChecks);
  }

  @Override
  protected void initialize(AbstractGoogleClientRequest<?> httpClientRequest) throws java.io.IOException {
    super.initialize(httpClientRequest);
  }

  /**
   * An accessor for creating requests from the Elections collection.
   *
   * <p>The typical use is:</p>
   * <pre>
   *   {@code CivicInfo civicinfo = new CivicInfo(...);}
   *   {@code CivicInfo.Elections.List request = civicinfo.elections().list(parameters ...)}
   * </pre>
   *
   * @return the resource collection
   */
  public Elections elections() {
    return new Elections();
  }

  /**
   * The "elections" collection of methods.
   */
  public class Elections {

    /**
     * List of available elections to query.
     *
     * Create a request for the method "elections.electionQuery".
     *
     * This request holds the parameters needed by the the civicinfo server.  After setting any optional
     * parameters, call the {@link ElectionQuery#execute()} method to invoke the remote operation.
     *
     * @return the request
     */
    public ElectionQuery electionQuery() throws java.io.IOException {
      ElectionQuery result = new ElectionQuery();
      initialize(result);
      return result;
    }

    public class ElectionQuery extends CivicInfoRequest<com.google.api.services.civicinfo.model.ElectionsQueryResponse> {

      private static final String REST_PATH = "elections";

      /**
       * List of available elections to query.
       *
       * Create a request for the method "elections.electionQuery".
       *
       * This request holds the parameters needed by the the civicinfo server.  After setting any
       * optional parameters, call the {@link ElectionQuery#execute()} method to invoke the remote
       * operation. <p> {@link ElectionQuery#initialize(AbstractGoogleClientRequest)} must be called to
       * initialize this instance immediately after invoking the constructor. </p>
       *
       * @since 1.13
       */
      protected ElectionQuery() {
        super(CivicInfo.this, "GET", REST_PATH, null, com.google.api.services.civicinfo.model.ElectionsQueryResponse.class);
      }

      @Override
      public com.google.api.client.http.HttpResponse executeUsingHead() throws java.io.IOException {
        return super.executeUsingHead();
      }

      @Override
      public com.google.api.client.http.HttpRequest buildHttpRequestUsingHead() throws java.io.IOException {
        return super.buildHttpRequestUsingHead();
      }

      @Override
      public ElectionQuery setAlt(String alt) {
        return (ElectionQuery) super.setAlt(alt);
      }

      @Override
      public ElectionQuery setFields(String fields) {
        return (ElectionQuery) super.setFields(fields);
      }

      @Override
      public ElectionQuery setKey(String key) {
        return (ElectionQuery) super.setKey(key);
      }

      @Override
      public ElectionQuery setOauthToken(String oauthToken) {
        return (ElectionQuery) super.setOauthToken(oauthToken);
      }

      @Override
      public ElectionQuery setPrettyPrint(Boolean prettyPrint) {
        return (ElectionQuery) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public ElectionQuery setQuotaUser(String quotaUser) {
        return (ElectionQuery) super.setQuotaUser(quotaUser);
      }

      @Override
      public ElectionQuery setUserIp(String userIp) {
        return (ElectionQuery) super.setUserIp(userIp);
      }

    }
    /**
     * Looks up information relevant to a voter based on the voter's registered address.
     *
     * Create a request for the method "elections.voterInfoQuery".
     *
     * This request holds the parameters needed by the the civicinfo server.  After setting any optional
     * parameters, call the {@link VoterInfoQuery#execute()} method to invoke the remote operation.
     *
     * @param electionId The unique ID of the election to look up. A list of election IDs can be obtained
     *        at.https://www.googleapis.com/civicinfo/{version}/elections
     * @param content the {@link com.google.api.services.civicinfo.model.VoterInfoRequest}
     * @return the request
     */
    public VoterInfoQuery voterInfoQuery(Long electionId, com.google.api.services.civicinfo.model.VoterInfoRequest content) throws java.io.IOException {
      VoterInfoQuery result = new VoterInfoQuery(electionId, content);
      initialize(result);
      return result;
    }

    public class VoterInfoQuery extends CivicInfoRequest<com.google.api.services.civicinfo.model.VoterInfoResponse> {

      private static final String REST_PATH = "voterinfo/{electionId}/lookup";

      /**
       * Looks up information relevant to a voter based on the voter's registered address.
       *
       * Create a request for the method "elections.voterInfoQuery".
       *
       * This request holds the parameters needed by the the civicinfo server.  After setting any
       * optional parameters, call the {@link VoterInfoQuery#execute()} method to invoke the remote
       * operation. <p> {@link VoterInfoQuery#initialize(AbstractGoogleClientRequest)} must be called to
       * initialize this instance immediately after invoking the constructor. </p>
       *
       * @param electionId The unique ID of the election to look up. A list of election IDs can be obtained
     *        at.https://www.googleapis.com/civicinfo/{version}/elections
       * @param content the {@link com.google.api.services.civicinfo.model.VoterInfoRequest}
       * @since 1.13
       */
      protected VoterInfoQuery(Long electionId, com.google.api.services.civicinfo.model.VoterInfoRequest content) {
        super(CivicInfo.this, "POST", REST_PATH, content, com.google.api.services.civicinfo.model.VoterInfoResponse.class);
        this.electionId = Preconditions.checkNotNull(electionId, "Required parameter electionId must be specified.");
      }

      @Override
      public VoterInfoQuery setAlt(String alt) {
        return (VoterInfoQuery) super.setAlt(alt);
      }

      @Override
      public VoterInfoQuery setFields(String fields) {
        return (VoterInfoQuery) super.setFields(fields);
      }

      @Override
      public VoterInfoQuery setKey(String key) {
        return (VoterInfoQuery) super.setKey(key);
      }

      @Override
      public VoterInfoQuery setOauthToken(String oauthToken) {
        return (VoterInfoQuery) super.setOauthToken(oauthToken);
      }

      @Override
      public VoterInfoQuery setPrettyPrint(Boolean prettyPrint) {
        return (VoterInfoQuery) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public VoterInfoQuery setQuotaUser(String quotaUser) {
        return (VoterInfoQuery) super.setQuotaUser(quotaUser);
      }

      @Override
      public VoterInfoQuery setUserIp(String userIp) {
        return (VoterInfoQuery) super.setUserIp(userIp);
      }

      /**
     * The unique ID of the election to look up. A list of election IDs can be obtained
     * at.https://www.googleapis.com/civicinfo/{version}/elections
     */
      @com.google.api.client.util.Key
      private Long electionId;

      /** The unique ID of the election to look up. A list of election IDs can be obtained
     at.https://www.googleapis.com/civicinfo/{version}/elections
       */
      public Long getElectionId() {
        return electionId;
      }

      /**
     * The unique ID of the election to look up. A list of election IDs can be obtained
     * at.https://www.googleapis.com/civicinfo/{version}/elections
     */
      public VoterInfoQuery setElectionId(Long electionId) {
        this.electionId = electionId;
        return this;
      }

      /** If set to true, only data from official state sources will be returned. */
      @com.google.api.client.util.Key
      private Boolean officialOnly;

      /** If set to true, only data from official state sources will be returned. [default: false]
       */
      public Boolean getOfficialOnly() {
        return officialOnly;
      }

      /** If set to true, only data from official state sources will be returned. */
      public VoterInfoQuery setOfficialOnly(Boolean officialOnly) {
        this.officialOnly = officialOnly;
        return this;
      }

      /**
       * Convenience method that returns only {@link Boolean#TRUE} or {@link Boolean#FALSE}.
       *
       * <p>
       * Boolean properties can have four possible values:
       * {@code null}, {@link com.google.api.client.util.Data#NULL_BOOLEAN}, {@link Boolean#TRUE}
       * or {@link Boolean#FALSE}.
       * </p>
       *
       * <p>
       * This method returns {@link Boolean#TRUE} if the default of the property is {@link Boolean#TRUE}
       * and it is {@code null} or {@link com.google.api.client.util.Data#NULL_BOOLEAN}.
       * {@link Boolean#FALSE} is returned if the default of the property is {@link Boolean#FALSE} and
       * it is {@code null} or {@link com.google.api.client.util.Data#NULL_BOOLEAN}.
       * </p>
       *
       * <p>
       * If set to true, only data from official state sources will be returned.
       * </p>
       */
      public boolean isOfficialOnly() {
        if (officialOnly == null || officialOnly == com.google.api.client.util.Data.NULL_BOOLEAN) {
          return false;
        }
        return officialOnly;
      }

    }

  }

  /**
   * Builder for {@link CivicInfo}.
   *
   * <p>
   * Implementation is not thread-safe.
   * </p>
   *
   * @since 1.3.0
   */
  public static final class Builder extends AbstractGoogleJsonClient.Builder {

    /**
     * Returns an instance of a new builder.
     *
     * @param transport HTTP transport
     * @param jsonFactory JSON factory
     * @param httpRequestInitializer HTTP request initializer or {@code null} for none
     * @since 1.7
     */
    public Builder(HttpTransport transport, JsonFactory jsonFactory,
        HttpRequestInitializer httpRequestInitializer) {
      super(
          transport,
          jsonFactory,
          DEFAULT_ROOT_URL,
          DEFAULT_SERVICE_PATH,
          httpRequestInitializer,
          false);
    }

    /** Builds a new instance of {@link CivicInfo}. */
    @Override
    public CivicInfo build() {
      return new CivicInfo(getTransport(),
          getHttpRequestInitializer(),
          getRootUrl(),
          getServicePath(),
          getObjectParser(),
          getGoogleClientRequestInitializer(),
          getApplicationName(),
          getSuppressPatternChecks());
    }

    @Override
    public Builder setRootUrl(String rootUrl) {
      return (Builder) super.setRootUrl(rootUrl);
    }

    @Override
    public Builder setServicePath(String servicePath) {
      return (Builder) super.setServicePath(servicePath);
    }

    @Override
    public Builder setHttpRequestInitializer(HttpRequestInitializer httpRequestInitializer) {
      return (Builder) super.setHttpRequestInitializer(httpRequestInitializer);
    }

    @Override
    public Builder setApplicationName(String applicationName) {
      return (Builder) super.setApplicationName(applicationName);
    }

    @Override
    public Builder setSuppressPatternChecks(boolean suppressPatternChecks) {
      return (Builder) super.setSuppressPatternChecks(suppressPatternChecks);
    }

    /**
     * Set the {@link CivicInfoRequestInitializer}.
     *
     * @since 1.12
     */
    public Builder setCivicInfoRequestInitializer(
        CivicInfoRequestInitializer civicinfoRequestInitializer) {
      return (Builder) super.setGoogleClientRequestInitializer(civicinfoRequestInitializer);
    }

    @Override
    public Builder setGoogleClientRequestInitializer(
        GoogleClientRequestInitializer googleClientRequestInitializer) {
      return (Builder) super.setGoogleClientRequestInitializer(googleClientRequestInitializer);
    }
  }
}
